import pytest
from rocket_releaser.graphql import GraphQL, BadReturnStatus

g = GraphQL("")


def test_run_query_returns_json_when_good_status_code(mocker):
    class MockResponse:
        status_code = 200

        @staticmethod
        def json():
            return "mock json"

    mocker.patch("requests.post", return_value=MockResponse())

    response = g.run_query("")

    assert response == "mock json"


def test_run_query_raises_error_when_bad_status_code(mocker):
    class MockResponse:
        status_code = 500

        @staticmethod
        def json():
            return "mock json"

    mocker.patch("requests.post", return_value=MockResponse())

    with pytest.raises(BadReturnStatus):
        response = g.run_query("")



import logging
import re
from typing import List

import github3
import jira

from .prs import PRs

logger = logging.getLogger(__name__)


class TicketLabeler:
    """
    class for labeling github/jira pr's/tickets
    """

    # you can find more transition id's with the following code:
    """
    import jira
    import json
    jira = jira.JIRA('https://yourJira.atlassian.net', basic_auth=('your_account@company.com', 'JIRA SECRET'))
    issue = jira.issue('DEV-91') # replace this number with issue you want to get availible transitions for
    # keep in mind isssues in different states may have different transitions!
    transitions=[(t['name'], t['id']) for t in jira.transitions(issue)]
    d = dict(transitions)
    print(json.dumps(d, indent=4 ,sort_keys=True))
    """

    TRANSITION_IDS = {
        "Backlog": "871",
        "Blocked": "831",
        "Close": "851",
        "Code Review": "771",
        "Deployed": "861",
        "Ready to Test": "841",
        "Reopen Issue": "811",
        "Start Progress": "801",
        "Start Testing": "791",
        "Stop progress": "821",
    }

    TRANSITION_KEYWORDS = [
        "close",
        "closes",
        "closed",
        "fix",
        "fixes",
        "fixed",
        "resolve",
        "resolves",
        "resolved",
    ]

    JIRA_INFO_RE = re.compile(
        r"""
        (?P<transition>{transition_kw_options})?  # optional keywords that mark Jira transitions
        \s*                                       # optional whitespace character
        (?P<issue>\[*[A-Z]{{2,}}-\d+\]*)          # Jira issue name. Match examples: [ENG-123], ENG-123, DIV-1234
    """.format(
            transition_kw_options="|".join(TRANSITION_KEYWORDS)
        ),
        flags=re.VERBOSE | re.IGNORECASE,
    )

    PREVIEW_ENV_NAME: str = "preview"
    STAGING_ENV_NAME: str = "staging"
    PRODUCTION_ENV_NAME: str = "production"

    def __init__(
        self,
        githubToken: str,
        pull_request_dicts: List[dict],
        repo_owner: str,
        repo_name: str,
        jira_token: str = "",
        jira_username: str = "",
        jira_url: str = "",
    ):
        """
        :param githubToken: GitHub oauth githubToken
        """
        self.githubToken = githubToken
        self.pull_request_dicts = pull_request_dicts
        self.repo_owner = repo_owner
        self.repo_name = repo_name
        self.jira_token = jira_token
        self.jira_url = jira_url
        self.gh = github3.GitHub(token=self.githubToken)

        # documentation claims you need to use username/password combo but username/token works as well
        if jira_token:
            self.jira = jira.JIRA(jira_url, basic_auth=(jira_username, self.jira_token))

    def label_tickets(self, env_name: str, vpc_name: str, dry_run=False) -> int:
        """
        labels github pr's and associated jira tickets with env_name.
        Note that if env_name matches self.PRODUCTION_ENV_NAME the jira issue will be closed

        :param env_name: the name of the vpc to label the tickets/issues with
        :return: number of jira tickets found
        """
        if dry_run:
            logger.info("Dry run - not actually making any changes")

        jira_ticket_name_list = []

        label: str = f"{vpc_name}({env_name})" if vpc_name != env_name else env_name

        for pr in self.pull_request_dicts:

            title = pr.get("title")
            pr_num = pr.get("number")

            try:
                logger.info(
                    f"labeling pr #{pr_num} {title} at "
                    f"https://github.com/{self.repo_owner}/{self.repo_name}/pull/{pr_num} with {label}"
                )
                if not dry_run:
                    self.label_pr_or_issue(pr, label)

            except github3.exceptions.GitHubException:
                logger.exception("Error during labeling: ")

            if not self.jira_token:
                continue

            jira_ticket_maps = []
            jira_ticket_maps_title = self.get_jira_ticket_maps(pr.get("title", ""))
            jira_ticket_maps_body = self.get_jira_ticket_maps(pr.get("body", ""))

            for jira_ticket_map_title in jira_ticket_maps_title:
                jira_ticket_maps.append(jira_ticket_map_title)

            for jira_ticket_map_body in jira_ticket_maps_body:
                if jira_ticket_map_body.get("transition"):
                    jira_ticket_maps.append(jira_ticket_map_body)
                # we ignore tickets without transition in body because they may be unrelated
                # ex: "this story is similar to ENG-4235" ~ we dont want to label ENG-4235

            if not jira_ticket_maps:
                logger.warning(f"couldnt find jira # in pr #{pr_num} {title}")
                continue

            # ugly hack ~ by converting to dict & back we ensure we dont have duplicate issues
            # (same issue may be mentioned in title and again in body)
            jira_ticket_maps = list(
                {map["issue"]: map for map in jira_ticket_maps}.values()
            )

            for jira_ticket_map in jira_ticket_maps:
                jira_ticket_name = jira_ticket_map.get("issue")
                transition_kw = jira_ticket_map.get("transition")

                try:
                    if jira_ticket_name not in jira_ticket_name_list:
                        jira_ticket_name_list.append(jira_ticket_name)
                    logger.info(
                        f"labeling jira ticket at {self.jira_url}/browse/{jira_ticket_name}"
                        f" with {label}"
                    )
                    issue = self.jira.issue(jira_ticket_name)

                    # a ticket may have more than one PR
                    # so we only transition if final PR
                    if transition_kw:

                        if env_name == self.PREVIEW_ENV_NAME:
                            logger.info(
                                "env_name matches preview - making sure if ticket is still in progress "
                                "its marked in review"
                            )
                            if not dry_run:
                                self.mark_in_review_jira_ticket(issue)

                        # we don't have enough qa currently to test everything before it goes to prod
                        # so we don't close the ticket when it hits prod in case it still needs testing
                        # elif env_name == self.PRODUCTION_ENV_NAME:
                        #     logger.info('env_name matches production - closing jira ticket')
                        #     if not dry_run:
                        #         self.mark_deployed_jira_ticket(issue)

                    if not dry_run:
                        # this HAS to be last because you cant add labels to closed issue
                        self.label_jira_ticket(issue, jira_ticket_name, label)

                except (jira.exceptions.JIRAError, ValueError):
                    logger.exception("error with " + str(title))

        return len(jira_ticket_name_list)

    @staticmethod
    def get_jira_ticket_maps(search_string: str) -> List[dict]:
        """
        Looks for all Jira related information in a string, including transition keywords and ticket number.

        Will match the following strings:
            * '[ENG-123]'
            * 'ENG-123'
            * 'eng-12345'
            * 'closes DEV-1234`
            * 'Fixes DS-123'

        Note: The 'issue' value is always returned _without_ brackets.

        :param search_string: str that will be searched for all Jira information.
        :return: List of dictionaries, each having the keys 'issue' and 'transition'
        """
        cleaned_info_list = []
        for match_object in TicketLabeler.JIRA_INFO_RE.finditer(search_string):
            info_dict = match_object.groupdict()

            # lowercase the transition keyword if it is found
            transition = info_dict.get("transition")
            if transition:
                transition = transition.lower()

            # remove brackets and uppercase the issue name
            issue = info_dict.get("issue", "").replace("[", "").replace("]", "").upper()

            # edge case: 1-on-1 looks like a issue to regex, so we filter that out
            # alternatively we could require brackets surrounding issue
            if issue == "ON-1":
                continue

            cleaned_dict = {"transition": transition, "issue": issue}
            cleaned_info_list.append(cleaned_dict)

        return cleaned_info_list

    def label_pr_or_issue(self, pr: dict, label: str):
        """
        :param pr: dict with number key mapping to a string
        """
        pr_num = pr.get("number")
        issue = self.gh.issue(self.repo_owner, self.repo_name, pr_num)
        issue.add_labels(label)

    def label_jira_ticket(self, issue, ticket_name, label):

        if " " in label:
            raise ValueError("labels can't have spaces!")

        if label not in issue.fields.labels:
            issue.update(fields={"labels": issue.fields.labels + [label]})

        return issue

    def mark_deployed_jira_ticket(self, issue: jira.Issue):
        """
        Closes w/ comment then marks as deployed
        """
        if issue is None:
            raise TypeError("issue is None - issue should be of type jira.Issue")

        if (
            issue.fields.status.name != "Closed"
            and issue.fields.status.name != "Deployed"
        ):
            self.jira.transition_issue(
                issue,
                self.TRANSITION_IDS["Close"],
                fields={"resolution": {"name": "Done"}},
                comment="auto transitioned by deploy",
            )

        if issue.fields.status.name != "Deployed":
            # Note that you can't comment when transitioning to Deployed status
            self.jira.transition_issue(
                issue, self.TRANSITION_IDS["Deployed"], comment=""
            )

    def mark_ready_test_jira_ticket(self, issue: jira.Issue):
        """
        :param issue: jira Issue
        """
        if issue is None:
            raise TypeError("issue is None - issue should be of type jira.Issue")

        if issue.fields.status.name in ("Reopened", "Open", "In Progress", "In Review"):
            self.jira.transition_issue(issue, self.TRANSITION_IDS["Ready to Test"])

    def mark_in_review_jira_ticket(self, issue: jira.Issue):
        """
        :param issue: jira Issue
        """
        if issue is None:
            raise TypeError("issue is None - issue should be of type jira.Issue")

        if issue.fields.status.name in ("Reopened", "Open", "In Progress"):
            self.jira.transition_issue(issue, self.TRANSITION_IDS["Code Review"])



from os import path
import tempfile
from random import randint
import shutil
import subprocess
from git import Repo
from rocket_releaser.shas import branch_exists, SHAs

cherry_pick_commit_msg = """commit 3f9ba302b9d440caba0fc2ba1f19e3614e46a7f1
feature

(cherry picked from commit c8e9114beabca79e4497f9ea40499c80cebe902a)"""

# These are 20 actual commits from fifteen5 repo
commit_msgs = """commit 6bc22dacce6aa6d036ca58bcbed23aceff61609e
Merge pull request #22344 from 15five/ENG-1357-Question-templates-not-cloned-appropriately

Clone questions templates correctly [ENG-1357]
commit d59a711dbc5278d35dff1c926ece789219e4aef4
Merge pull request #22402 from 15five/dev

__2019-03-05.0 - to staging
commit 90b079b9cf0f2d28fec37387d8f45f13ec38e829
Fix my team filtering in BSRs

commit 0fa63c345191896add555d1d2fb0c56023ab4838
Merge pull request #22407 from 15five/hotfix-staging-2019-03-05.0

__2019-03-05.1 - to staging (hotfix)
commit 0640274e4d0ddabc7ccc8e2a5d70b75ae758b946
Better SCIM logging

commit 094b0de3f41c235e2a3d5ac29475c9376aae715b
Merge pull request #22414 from 15five/staging-hf

__2019-03-05.2 - to staging (hotfix)
commit 8fb0524f8011e39c27afd2e37965df7123a20b58
Exclude smart groups when saving UserEditForm

[ENG-1546]
"""


def commit_random_file(branch="master"):
    # see https://gitpython.readthedocs.io/en/stable/tutorial.html
    filename = str(randint(0, 999999999))
    new_file_path = path.join(repo.working_tree_dir, filename)
    open(new_file_path, "wb").close()  # create new file in working tree
    repo.index.add([new_file_path])  # add it to the index
    # Commit the changes to deviate masters history
    return repo.index.commit("Added " + filename)


tmp_dirpath = None
repo = None
first_commit = None
shas = None


def setup_module():
    global repo
    global first_commit
    global shas
    global tmp_dirpath
    tmp_dirpath = tempfile.mkdtemp()
    repo = Repo.init(tmp_dirpath)
    repo.git.config("user.email", "test_user@example.com")
    repo.git.config("user.name", "test_user")
    first_commit = commit_random_file()
    shas = SHAs(tmp_dirpath)


def test_get_shas_internal():
    shas_list = shas._get_shas(commit_msgs)
    assert len(shas_list) == 7


def test_cherry_picked_sha():
    shas_list = shas._get_shas(cherry_pick_commit_msg)
    assert len(shas_list) == 2
    assert shas_list[0] == "3f9ba302b9d440caba0fc2ba1f19e3614e46a7f1"
    assert shas_list[1] == "c8e9114beabca79e4497f9ea40499c80cebe902a"


def test_get_shas_empty_when_no_changes():
    my_shas = shas.get_shas(first_commit.hexsha, first_commit.hexsha)
    assert not my_shas


def test_old_commit_in_shas():
    # a old commit should still be in shas even if chronologically before last deploy

    # create & checkout (-b) branch
    branch_name = "old_branch"
    repo.git.checkout("-b", branch_name)

    # create old commit
    old_commit = commit_random_file(branch=branch_name)

    # create newer commit (realistically this would be another merge, but doesn't matter)
    # we pretend user has deployed this commit
    repo.git.checkout("master")
    last_deployed_commit = commit_random_file()

    # merge old commit into master
    repo.git.merge(branch_name)

    # pretend user deployed "old" commit

    my_shas = shas.get_shas(last_deployed_commit.hexsha, repo.head.commit.hexsha)
    assert my_shas
    assert old_commit.hexsha in my_shas
    assert last_deployed_commit.hexsha not in my_shas
    # ^ already been deployed, we don't want to log release notes of this commit
    assert repo.head.commit.hexsha in my_shas
    # ^ sanity check


def test_not_in_range():
    assert not shas.get_shas("nonexistant sha", "nonexistant sha")


def test_branch_exists():
    assert branch_exists(tmp_dirpath, "master")
    assert not branch_exists(
        tmp_dirpath, "did-you-know-that-cashews-came-from-a-fruit?"
    )


def teardown_module():
    repo.close()



import slacker
import logging

logger = logging.getLogger(__name__)


def post_deployment_message_to_slack(slack_webhook_key, text):

    incoming_webhook_url = f"https://hooks.slack.com/services/{slack_webhook_key}"

    # We will continue to use Slacker for posting internal messages to webhooks.
    # There is no reason to switch to SlackClient. Alternatively, we could just use
    # the requests library directly.
    slack = slacker.Slacker("", incoming_webhook_url=incoming_webhook_url)

    display_name = "Deployment Team"
    icon_url = ":rocket:"

    slack.incomingwebhook.post(
        {
            "text": text,
            "username": display_name,
            "icon_url": icon_url,
            "icon_emoji": icon_url,
        }
    )



import argparse
import datetime
import logging
import re
import subprocess
from sys import stdout, argv
from typing import List

from .changelog import ChangeLog
from .prs import PRs
from .shas import branch_exists, SHAs
from .slack import post_deployment_message_to_slack
from .ticket_labeler import TicketLabeler

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def turn_changelog_into_string(
    pull_request_dicts: List[dict],
    env_name: str,
    from_revision: str,
    to_revision: str,
    num_jira_tickets: int,
    org_name: str,
    repo_name: str,
):

    changelog = ChangeLog(pull_request_dicts, org_name, repo_name).parse_bodies()

    link = (
        f"<https://github.com/{org_name}/{repo_name}/compare/{from_revision[:7]}...{to_revision[:7]}|"
        f"{from_revision[:7]}...{to_revision[:7]}>"
    )
    now_str = datetime.datetime.now().isoformat(" ")
    title_line = f"*{env_name.upper()} RELEASE* {now_str} ({link})"

    # assumes that env_name is the same as the branch name
    # currently for 15Five this assumption works (only difference is dev/preview but that doesnt get hotfixes)
    is_hotfix = any(
        [
            "hotfix" in pr.get("title", "").lower()
            and env_name.lower() in pr.get("title", "").lower()
            for pr in pull_request_dicts
        ]
    )

    if is_hotfix:
        title_line = ":fire: *HOTFIX* :fire: " + title_line

    messages = [
        title_line,
        f"{num_jira_tickets} jira tickets found.",
        f"{len(changelog.pull_request_dicts)} PRs found.",
    ]

    if from_revision == to_revision:
        messages.append(
            "Start and end commit hashes are the same - no changes for release notes to log"
        )
        return "\n".join(messages)

    attr_names_and_category_names = [
        ("noteworthy", "Noteworthy Changes"),
        ("features", "Features"),
        ("fixes", "Fixes"),
    ]

    for attr_name, category_name in attr_names_and_category_names:
        category_items = getattr(changelog, attr_name)
        if category_items:
            messages.append(f"\n*{category_name}*")
            messages.extend(category_items)

    if changelog.qa_notes and env_name.lower() in ("preview", "staging"):
        messages.append("\n*Notes for QA*")
        messages.extend(changelog.qa_notes)

    text = "\n".join(messages)

    return text


def release_notes(
    github_token: str,
    from_revision: str,
    to_revision: str,
    org_name: str,
    repo_name: str,
    repo_dir: str = None,
    search_branch: str = "master",
    slack_webhook_key: str = "",
    env_name: str = "prod",
    vpc_name: str = "prod",
    jira_token: str = "",
    jira_username: str = "",
    jira_url: str = "",
    label_tickets=True,
    verbose=False,
    dry_run=False,
    fetch_before=True,
):

    if not repo_dir:
        repo_dir = get_default_repo_dir()

    if not branch_exists(repo_dir, search_branch):
        raise ValueError(search_branch + " branch does not exist in " + repo_dir)

    logger.info(
        f"Pulling deploy SHAs from {search_branch} branch in {repo_dir}. {from_revision}...{to_revision}"
    )
    deploy_shas = SHAs(repo_dir, fetch_before).get_shas(
        from_revision, to_revision, branch=search_branch
    )

    logger.info(f"Pulling PR bodies from GitHub. Searching {len(deploy_shas)} SHAs.")
    prs = PRs(github_token, org_name, repo_name)

    # Shas might be associated with unmerged pr's if the pr rebased itself to include that sha
    # We only want merged pr's
    pull_request_dicts = [
        pr for pr in prs.pull_request_dicts(deploy_shas) if pr["merged"]
    ]

    num_jira_tickets: int = 0

    if label_tickets:
        ticket_labeler = TicketLabeler(
            github_token,
            pull_request_dicts,
            org_name,
            repo_name,
            jira_token,
            jira_username,
            jira_url,
        )
        num_jira_tickets = ticket_labeler.label_tickets(
            env_name, vpc_name, dry_run=dry_run
        )
        logger.info(f"labeled {num_jira_tickets} tickets")

    slack_text = turn_changelog_into_string(
        pull_request_dicts,
        env_name,
        from_revision,
        to_revision,
        num_jira_tickets,
        org_name,
        repo_name,
    )

    if verbose:
        print(slack_text)

    if dry_run:
        return slack_text

    if slack_webhook_key:
        logger.info(f"Pushing ChangeLog data to {env_name} Slack channel.")
        post_deployment_message_to_slack(slack_webhook_key, slack_text)
    else:
        logger.warning("no slack webhook key. Not pushing to slack.")

    logger.info("Done.")

    return slack_text


def get_default_repo_dir():
    try:
        get_repo_dir_command = ["git", "rev-parse", "--show-toplevel"]
        repo_dir: str = subprocess.check_output(get_repo_dir_command).decode("utf-8")
        return repo_dir.strip()
    except subprocess.CalledProcessError:
        raise ValueError("You are not in a git repo - please specify git repo")


def main(args: List[str]):
    parser = argparse.ArgumentParser()
    parser.add_argument("github_token", help="GitHub API token")
    parser.add_argument("from_revision")
    parser.add_argument("to_revision")
    parser.add_argument("org_name")
    parser.add_argument("repo_name")
    parser.add_argument(
        "-r",
        "--repo_dir",
        help=f"Defaults to current repo if you are inside a repository",
        default="",
    )
    parser.add_argument(
        "-b", "--search_branch", help='Default is "master"', default="master"
    )
    parser.add_argument(
        "-s",
        "--slack_webhook_key",
        help='Eg. "ABC457E/FElF56789FE/FLIELAJFLKAJLKEFFE"',
        default="",
    )
    parser.add_argument("-e", "--env_name", help='Eg. "prod"', default="prod")
    parser.add_argument(
        "-V",
        "--vpc_name",
        help='Eg. "prod_1". If you dont have a VPC use the same value as env_name.',
        default="prod",
    )
    parser.add_argument(
        "-l",
        "--dont_label_tickets",
        help="If you skip labeling tickets the jira token won't matter",
        action="store_false",
        dest="label_tickets",
        default=True,
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        dest="verbose",
        default=False,
    )
    parser.add_argument(
        "-d",
        "--dry_run",
        action="store_true",
        dest="dry_run",
        default=False,
    )
    parser.add_argument(
        "--skip-fetch",
        action="store_false",
        dest="fetch_before",
        default=True,
    )
    parser.add_argument("--jira_token", help="Jira API token", default="")
    parser.add_argument("--jira_username", help='Eg. "bob@company.com"', default="")
    parser.add_argument(
        "--jira_url", help='Eg. "https://company.atlassian.net"', default=""
    )

    parsed_args = parser.parse_args(args)

    logger.info(
        "Running release notes script with following options: "
        + str(
            {
                "github_token": "CENSORED",
                "from_revision": parsed_args.from_revision,
                "to_revision": parsed_args.to_revision,
                "org_name": parsed_args.org_name,
                "repo_name": parsed_args.repo_name,
                "repo_dir": parsed_args.repo_dir,
                "search_branch": parsed_args.search_branch,
                "slack_webhook_key": "CENSORED",
                "env_name": parsed_args.env_name,
                "vpc_name": parsed_args.vpc_name,
                "label_tickets": parsed_args.label_tickets,
                "verbose": parsed_args.verbose,
                "dry_run": parsed_args.dry_run,
                "fetch_before": parsed_args.fetch_before,
                "jira_token": "CENSORED",
                "jira_username": parsed_args.jira_username,
                "jira_url": parsed_args.jira_url,
            }
        )
    )

    if parsed_args.verbose:
        logger.setLevel(logging.DEBUG)

    return release_notes(
        parsed_args.github_token,
        parsed_args.from_revision,
        parsed_args.to_revision,
        parsed_args.org_name,
        parsed_args.repo_name,
        repo_dir=parsed_args.repo_dir,
        search_branch=parsed_args.search_branch,
        slack_webhook_key=parsed_args.slack_webhook_key,
        env_name=parsed_args.env_name,
        vpc_name=parsed_args.vpc_name,
        jira_token=parsed_args.jira_token,
        jira_username=parsed_args.jira_username,
        jira_url=parsed_args.jira_url,
        label_tickets=parsed_args.label_tickets,
        verbose=parsed_args.verbose,
        dry_run=parsed_args.dry_run,
    )



import requests


class BadReturnStatus(Exception):
    pass


class GraphQL:
    def __init__(self, base_api_uri, token: str = ""):
        self.base_api_uri = base_api_uri
        self.headers = {"Authorization": "Bearer " + token}

    def run_query(self, query, variables={}):
        request = requests.post(
            self.base_api_uri,
            json={"query": query, "variables": variables},
            headers=self.headers,
        )

        if request.status_code == 200:
            return request.json()
        else:
            raise BadReturnStatus(
                "Query failed to run by returning code of {}. {}".format(
                    request.status_code, query
                )
            )



import logging
from typing import List

from .graphql import GraphQL

logger = logging.getLogger(__name__)


class PRs:
    def __init__(self, token: str, repo_owner: str, repo_name: str):
        """
        :param token: GitHub oauth token
        """
        self.token = token
        self.repo_owner = repo_owner
        self.repo_name = repo_name

        self._request_dicts = None

    def clear_cache(self):
        self._request_dicts = None

    def pull_request_dicts(self, deploy_shas: List[str] = []) -> List[dict]:
        """
        gets PR's associated with deploy shas. Cached so calling multiple times won't result in unnecessary API calls.
        """

        # number of associatedPullRequests to pull is entirely arbitrary
        # 99% of cases it should only be 1 anyways
        query = """
query associatedPRs($sha: String, $repo: String!, $owner: String!){
  repository(name: $repo, owner: $owner) {
    commit: object(expression: $sha) {
      ... on Commit {
        associatedPullRequests(first:5){
          edges{
            node{
              title
              number
              body
              merged
            }
          }
        }
      }
    }
  }
}
"""

        if self._request_dicts is None:

            pull_request_dicts: List[dict] = []
            seen = {}

            graph_ql = GraphQL("https://api.github.com/graphql", self.token)

            deploy_shas = set(deploy_shas)  # get rid of duplicates

            for sha in deploy_shas:
                result = graph_ql.run_query(
                    query,
                    {"sha": sha, "repo": self.repo_name, "owner": self.repo_owner},
                )

                try:
                    # 'errors' key is only present in result if there is an error
                    logger.warning(
                        f"error with sha {sha}: " + result["errors"][0]["message"]
                    )
                    continue
                except KeyError:
                    pass

                try:
                    for edge in result["data"]["repository"]["commit"][
                        "associatedPullRequests"
                    ]["edges"]:
                        pr = edge["node"]
                        pr_num = pr["number"]

                        # for logging we record sha we got to this pr from
                        pr["deploy_sha"] = sha

                        # two commits may reference the same PR
                        if pr_num not in seen:
                            pull_request_dicts.append(pr)
                            seen[pr_num] = True
                except TypeError:
                    # this can happen if commit is cherry-picked from local commit not in repo
                    logger.warning(
                        "commit %s not found or has no associated PRs", sha, exc_info=1
                    )

            self._request_dicts = pull_request_dicts

        return self._request_dicts



from rocket_releaser.changelog import ChangeLog


def test_release_bodies_should_recognize_releases():
    pull_request_dict = {"number": 12, "body": "RELEASES\n closes [ENG-413]"}
    assert ChangeLog([pull_request_dict], "org_name", "repo_name").release_bodies


def test_release_bodies_should_recognize_not_releases():
    pull_request_dict = {"number": 12, "body": "bla bla bla"}
    assert not ChangeLog([pull_request_dict], "org_name", "repo_name").release_bodies


def test_parse_bodies_should_only_pick_up_release_block():
    pull_request_dict = {"number": 12, "body": "RELEASES\n closes [ENG-413]\n\nIGNORE"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert "IGNORE" not in c.features or c.fixes or c.noteworthy or c.qa_notes


def test_parse_bodies_should_recognize_noteworthy():
    pull_request_dict = {"number": 12, "body": "RELEASES\n closes [ENG-413]"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert c.noteworthy


def test_parse_bodies_should_recognize_not_noteworthy():
    pull_request_dict = {"number": 12, "body": "RELEASES\n bla bla bla"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert not c.noteworthy


# this test currently failing due to bug in code
# where line is line is recognized as noteworthy instead of fix
def test_parse_bodies_should_recognize_fix():
    pull_request_dict = {"number": 12, "body": "RELEASES\n fixes [ENG-413]"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert c.fixes


def test_parse_bodies_should_recognize_no_fix():
    pull_request_dict = {"number": 12, "body": "RELEASES\n closes [ENG-413]"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert not c.fixes


def test_parse_bodies_should_recognize_features():
    pull_request_dict = {"number": 12, "body": "RELEASES\n more foobar"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert c.features


def test_parse_bodies_should_recognize_no_features():
    pull_request_dict = {
        "number": 12,
        "body": "RELEASES\ncloses [ENG-413]\nfixes error",
    }

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert not c.features


def test_parse_bodies_should_recognize_qa_notes():
    pull_request_dict = {"number": 12, "body": "RELEASES bla\n\nqa\n more foobar"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert not c.noteworthy
    assert not c.fixes
    assert not c.features
    assert c.qa_notes


def test_parse_bodies_should_recognize_no_qa_notes():
    pull_request_dict = {"number": 12, "body": "RELEASES\n\nbla"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert not c.qa_notes


def test_parse_bodies_should_not_split_qa_notes():
    pull_request_dict = {"number": 12, "body": "RELEASES\n\nQA:\ndo this\n and that"}

    c = ChangeLog([pull_request_dict], "org_name", "repo_name")
    c.parse_bodies()
    assert len(c.qa_notes) == 1


def test_parse_bodies_should_add_jira_link():
    pull_request_dict = {"number": 12, "body": "RELEASES\n[DS-435]"}

    c = ChangeLog(
        [pull_request_dict],
        "org_name",
        "repo_name",
        jira_url="https://company.atlassian.net",
    )
    c.parse_bodies()
    assert "atlassian" in c.noteworthy[0]



from rocket_releaser.ticket_labeler import TicketLabeler
import pytest
from pytest_mock import MockFixture
from unittest.mock import Mock
from rocket_releaser.prs import PRs


@pytest.fixture(autouse=True)
def mock_reqs(mocker):
    mocker.patch("github3.GitHub")
    mocker.patch("jira.JIRA")


def test_no_jira_ticket_mentioned(mocker: MockFixture):
    pull_request_dicts = [
        {
            "number": 5,
            "title": "removing middle finger emoji",
            "body": "extra text goes here.",
        }
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 0


def test_one_jira_tickets_with_number_in_title(mocker: MockFixture):
    pull_request_dicts = [
        {"number": 5, "title": "removing middle finger emoji [ENG-666]"}
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 1


def test_one_jira_tickets_with_number_in_body(mocker: MockFixture):
    pull_request_dicts = [
        {
            "number": 5,
            "title": "removing middle finger emoji",
            "body": "removing middle finger emoji fixes [ENG-666]",
        }
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 1


def test_random_jira_tickets_in_body_are_not_counted(mocker: MockFixture):
    pull_request_dicts = [
        {
            "number": 5,
            "title": "removing middle finger emoji",
            "body": "removing middle finger emoji. Similar to other ticket [ENG-666]",
        }
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 0


# todo: assert preview/staging/prod tickets were transitioned


def test_one_jira_ticket_preview(mocker: MockFixture):
    pull_request_dicts = [{"number": 5, "title": "closes [ENG-666]"}]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("preview", "preview")
    assert numTickets == 1


def test_one_jira_ticket_staging(mocker: MockFixture):
    pull_request_dicts = [{"number": 5, "title": "fixes [ENG-666]"}]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("staging", "staging")
    assert numTickets == 1


def test_one_jira_ticket_production(mocker: MockFixture):
    pull_request_dicts = [{"number": 5, "title": "closes  [ENG-666]"}]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("production", "production")
    assert numTickets == 1


def test_multiple_jira_tickets_with_number_in_title(mocker: MockFixture):
    pull_request_dicts = [
        {"number": 5, "title": "removing middle finger emoji [ENG-666]"},
        {"number": 6, "title": "More foobar [ENG-667]"},
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 2


def test_multiple_jira_tickets_with_number_in_body(mocker: MockFixture):
    pull_request_dicts = [
        {
            "number": 5,
            "title": "removing middle finger emoji",
            "body": "removing middle finger emoji fixes [ENG-666]",
        },
        {"number": 6, "title": "More foobar", "body": "More foobar closes [ENG-667]"},
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 2


def test_one_jira_ticket_but_two_prs(mocker: MockFixture):
    pull_request_dicts = [
        {"number": 5, "title": "removing middle finger emoji"},
        {"number": 6, "title": "More foobar [ENG-667]"},
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 1


def test_one_jira_referenced_in_both_title_and_body_only_returns_single_issue(
    mocker: MockFixture,
):
    pull_request_dicts = [
        {"number": 6, "title": "More foobar [ENG-667]", "body": "closes ENG-667"}
    ]

    t = TicketLabeler(
        "",
        pull_request_dicts,
        "",
        "",
        "fakeJiraToken",
        "bob@company.com",
        "https://foo.atlassian.net",
    )
    numTickets = t.label_tickets("", "")
    assert numTickets == 1


def test_get_jira_ticket_maps_for_empty_string():
    search_string = """
    Call me Ishmael. Some years ago--never mind how long precisely --having little or no money in
    my purse, and nothing particular to interest me on shore, I thought I would sail about a little and see the watery
    part of the world.
    """
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info == []


def test_get_jira_ticket_maps_for_string_without_jira_info():
    search_string = ""
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info == []


def test_get_jira_ticket_maps_for_one_on_ones():
    search_string = "bla bla bla 1-on-1"  # on-1 is not a actual ticket name
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info == []


def test_get_jira_ticket_maps_for_single_issue_string_without_brackets():
    search_string = "Some text goes ENG-123 here"
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info[0]["issue"] == "ENG-123"
    assert jira_ticket_info[0]["transition"] is None


def test_get_jira_ticket_maps_for_single_issue_string_with_brackets():
    search_string = "Some text goes [ENG-123] here"
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info[0]["issue"] == "ENG-123"
    assert jira_ticket_info[0]["transition"] is None


def test_get_jira_ticket_maps_for_single_issue_string_with_transition_keyword():
    search_string = "Some text goes closes ENG-123 here"
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info[0]["issue"] == "ENG-123"
    assert jira_ticket_info[0]["transition"] == "closes"


def test_get_jira_ticket_maps_for_single_issue_lowercase_string():
    search_string = "Some text goes [eng-123] here"
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info[0]["issue"] == "ENG-123"
    assert jira_ticket_info[0]["transition"] is None


def test_get_jira_ticket_maps_for_single_issue_with_mixed_case_string():
    search_string = "Closes Eng-123, fixing this ticket"
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info[0]["issue"] == "ENG-123"
    assert jira_ticket_info[0]["transition"] == "closes"


def test_get_jira_ticket_maps_for_multiple_issue_string():
    search_string = """
        Some text goes ENG-123 here
        RELEASES
        Fixes [DEV-723], after making some changes
        also linked to [ENG-4567]
        """
    jira_ticket_info = TicketLabeler.get_jira_ticket_maps(search_string)
    assert jira_ticket_info[0]["issue"] == "ENG-123"
    assert jira_ticket_info[0]["transition"] is None
    assert jira_ticket_info[1]["issue"] == "DEV-723"
    assert jira_ticket_info[1]["transition"] == "fixes"
    assert jira_ticket_info[2]["issue"] == "ENG-4567"
    assert jira_ticket_info[2]["transition"] is None



import re
import logging
from typing import List

logger = logging.getLogger(__name__)


class ChangeLog:
    def __init__(
        self, pull_request_dicts: List[dict], org_name: str, repo_name: str, jira_url=""
    ):
        self.pull_request_dicts = pull_request_dicts

        self.features = []
        self.fixes = []
        self.noteworthy = []
        self.qa_notes = []
        self.org_name = org_name
        self.repo_name = repo_name
        self.jira_url = jira_url

    @property
    def release_bodies(self):
        return [
            (pr.get("number"), pr.get("body"))
            for pr in self.pull_request_dicts
            if "release" in pr.get("body").lower()
        ]

    @staticmethod
    def is_noteworthy(line: str):
        return "[" in line

    @staticmethod
    def is_fix(line: str):
        line = line.lower()
        return "fixes [" in line or line.startswith("fix")

    @staticmethod
    def add_github_link(org_name: str, repo_name: str, line, pr_number):
        return f"{line} PR: <https://github.com/{org_name}/{repo_name}/pull/{pr_number}|{pr_number}>"

    @staticmethod
    def add_jira_link(line: str, jira_url: str):
        return re.sub(
            r"\[(\w+-\d+)\]",
            lambda match: f"<{jira_url}/browse/{match.group(1)}|{match.group(1)}>",
            line,
        )

    @staticmethod
    def linkify(org_name: str, repo_name: str, line: str, pr_number, jira_url=""):
        """add jira & github links"""

        if jira_url:
            line = ChangeLog.add_jira_link(line, jira_url)
        line = ChangeLog.add_github_link(org_name, repo_name, line, pr_number)
        return line

    @staticmethod
    def make_jira_id_bold(line: str):
        return line.replace("[", "*[").replace("]", "]*")

    def parse_bodies(self):
        for pr_number, body in self.release_bodies:
            # Remove HTML comments. Will not work in all cases.
            lines = re.sub("<!--.+?>", "", body, flags=re.DOTALL)
            lines = lines.strip().split("\n")
            # Clean lines up
            lines = list(map(lambda line_: line_.strip().lstrip("-").strip(), lines))

            fixes = ""
            noteworthy = ""
            features = ""
            collecting_releases = False
            for line in lines:
                if collecting_releases and not line:
                    # No more lines in RELEASES block
                    break

                if collecting_releases:
                    if self.is_fix(line):
                        fixes = fixes + " " + line
                    elif self.is_noteworthy(line):
                        line = self.make_jira_id_bold(line)
                        noteworthy = noteworthy + " " + line
                    else:
                        features = features + " " + line

                if not collecting_releases and line.lower().startswith("release"):
                    collecting_releases = True

            if fixes:
                self.fixes.append(
                    self.linkify(
                        self.org_name, self.repo_name, fixes, pr_number, self.jira_url
                    )
                )
            if noteworthy:
                self.noteworthy.append(
                    self.linkify(
                        self.org_name,
                        self.repo_name,
                        noteworthy,
                        pr_number,
                        self.jira_url,
                    )
                )
            if features:
                self.features.append(
                    self.linkify(
                        self.org_name,
                        self.repo_name,
                        features,
                        pr_number,
                        self.jira_url,
                    )
                )

            qa_notes = ""
            collecting_qa_notes = False
            for line in lines:
                if collecting_qa_notes and not line:
                    # No more lines in QA block
                    break

                if collecting_qa_notes:
                    qa_notes = qa_notes + " " + line

                if not collecting_qa_notes and line.lower().startswith("qa"):
                    collecting_qa_notes = True

            if qa_notes:
                self.qa_notes.append(
                    self.linkify(
                        self.org_name,
                        self.repo_name,
                        qa_notes,
                        pr_number,
                        self.jira_url,
                    )
                )

        return self



import subprocess
import logging
from os import path
from typing import List
import re

logger = logging.getLogger(__name__)


def branch_exists(repo_dir: str, branch_name: str):
    try:
        base_args = ["git", f'--git-dir={path.join(repo_dir, ".git")}']
        subprocess.check_call(
            base_args + ["show-ref", "--verify", "--quiet", "refs/heads/" + branch_name]
        )
        return True
    except subprocess.CalledProcessError:
        return False


class SHAs:
    def __init__(self, repo_dir: str, fetch_before: bool = True):
        """
        :param fetch_before: Whether to fetch branch to make sure you have it when calling for_branch.
        """

        self.fetch_before = fetch_before

        self.base_args = [
            "git",
            f'--git-dir={path.join(repo_dir, ".git")}',
            f"--work-tree={repo_dir}",
        ]

    def get_shas(
        self, from_revision: str, to_revision: str, branch: str = "master"
    ) -> List[str]:
        """
        Get SHAs from from_revision to to_revision.
        :param branch: Branch to pull commits from.
        """

        if self.fetch_before:
            fetch_args = self.base_args + [
                "fetch",
                "origin",
                f"{branch}:{branch}",
                "--update-head-ok",
            ]

            try:
                subprocess.check_output(fetch_args)
                logger.debug(f"Pulled branch with the following args: {fetch_args}")
            except subprocess.CalledProcessError:
                logger.exception("subprocess call failed")

        rev_list_args = self.base_args + [
            "rev-list",
            # https://stackoverflow.com/questions/7251477/what-are-the-differences-between-double-dot-and-triple-dot-in-git-dif
            # to be honest not sure what difference is between .. and ... when on same branch
            # but staying with ... to be on safe side because it includes more
            # I didn't find any differences when doing manual tests
            from_revision + "..." + to_revision,
            "--format=%B",  # raw body (unwrapped subject and body) https://git-scm.com/docs/git-rev-list
        ]

        try:
            commit_msgs = subprocess.check_output(rev_list_args).decode("utf-8")
        except subprocess.CalledProcessError:
            logger.exception("subprocess call failed")
            commit_msgs = ""

        logger.debug(f"rev-list commit messages: {commit_msgs}")

        shas = self._get_shas(commit_msgs)

        logger.debug(f"rev-list SHAs: {shas}")

        return shas

    def _get_shas(self, commit_msgs: str) -> List[str]:
        """returns shas, inlcuding cherry-picked shas"""

        shas = []

        for line in commit_msgs.split("\n"):

            line = line.rstrip("\r")

            if re.match(r"commit \w+$", line):
                shas.append(line.replace("commit ", ""))

            if line.startswith("(cherry picked from commit "):
                shas.append(line.replace("(cherry picked from commit ", "").rstrip(")"))

        return shas



#!/usr/bin/env python

from setuptools import setup, find_packages

VERSION = "0.1.1a"

setup(
    name="rocket_releaser",
    version=VERSION,
    description="Script for release notes and labeling upon deploys",
    long_description="See repo for README.md",
    url="https://github.com/15five/rocket_releaser",
    author="15Five",
    author_email="devops@15five.com",
    license="MIT",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.12",
    ],
    keywords="release-notes release-automation",
    packages=find_packages(),
    install_requires=["gitdb2>=4", "github3.py>=4", "jira>=3", "slacker"],
    python_requires=">=3.9",
)



from rocket_releaser import release_notes
import pytest
from pytest_mock import MockFixture
from unittest.mock import Mock
from os import environ


@pytest.fixture(autouse=True)
def mock_reqs(mocker):
    mocker.patch("github3.GitHub")
    mocker.patch("jira.JIRA")
    mocker.patch("rocket_releaser.prs.GraphQL")
    mocker.patch("rocket_releaser.slack.slacker")


def test_main():

    # arguments dont really matter here
    slack_text = release_notes.main(
        [
            "github token",
            "0782415",
            "8038fc3",
            "org_name",
            "repo_name",
            "--slack_webhook_key",
            "fake slack key",
            "--jira_username",
            "bob@company.com",
            "--jira_url",
            "https://company.atlassian.net",
        ]
    )

    # slack_text has variable time so we cant do exact comparison check :(

    # slack text should be present
    assert slack_text
    # slack text should say env of deployment
    assert "prod".upper() in slack_text
    # slack text should have link to github diff of commits
    assert "0782415" in slack_text and "8038fc3" in slack_text
    # because we mocked dependencies no PR's / tickets should be found
    assert "0 PRs found" in slack_text
    assert "0 jira tickets found" in slack_text


def test_release_notes_skips_unmerged_pr(mocker):
    mock_pr_1 = {
        "number": 1,
        "body": """
RELEASES
Fixes ENG-9999

QA
bla bla bla
        """,
        "merged": False,
    }

    mocker.patch("rocket_releaser.prs.PRs.pull_request_dicts", return_value=[mock_pr_1])
    slack_text = release_notes.release_notes(
        "github_token",
        "0782415",
        "8038fc3",
        "org_name",
        "repo_name",
        jira_token="jira_token",
        jira_username="",
        jira_url="",
        dry_run=True,
    )
    assert "0 PRs found" in slack_text


def test_turn_changelog_into_string_has_qa_notes():
    mock_pr_1 = {
        "number": 1,
        "body": """
RELEASES
Fixes ENG-9999

QA
bla bla bla
        """,
    }
    changelog_str = release_notes.turn_changelog_into_string(
        [mock_pr_1], "preview", "0782415", "8038fc3", 1, "org_name", "repo_name"
    )
    assert "Fixes" in changelog_str
    assert "Notes for QA" in changelog_str


def test_turn_changelog_into_string_detects_hotfix():
    mock_pr_1 = {
        "title": "__2019-07-05.0 - to staging (hotfix)",
        "number": 1,
        "body": "",
    }
    changelog_str = release_notes.turn_changelog_into_string(
        [mock_pr_1], "production", "0782415", "8038fc3", 1, "org_name", "repo_name"
    )
    first_line = changelog_str.split("\n")[0]
    assert "HOTFIX" not in first_line

    mock_pr_1 = {
        "title": "__2019-07-05.0 - to production (hotfix)",
        "number": 1,
        "body": "",
    }
    changelog_str = release_notes.turn_changelog_into_string(
        [mock_pr_1], "production", "0782415", "8038fc3", 1, "org_name", "repo_name"
    )
    first_line = changelog_str.split("\n")[0]
    assert "HOTFIX" in first_line


def test_turn_changelog_into_string_detects_when_commits_are_same():
    changelog_str = release_notes.turn_changelog_into_string(
        [], "preview", "", "", 55, "org_name", "repo_name"
    )
    assert "Start and end commit hashes are the same" in changelog_str


def test_turn_changelog_into_string_has_correct_num_jira_tickets():
    mock_pr_1 = {
        "number": 1,
        "body": """
RELEASES
Fixes ENG-9999

QA
bla bla bla
        """,
    }
    changelog_str = release_notes.turn_changelog_into_string(
        [mock_pr_1], "preview", "", "", 55, "org_name", "repo_name"
    )
    assert "55" in changelog_str



from rocket_releaser.prs import PRs
import pytest
from pytest_mock import MockFixture


class MockGraphQL:

    val = {
        "data": {
            "repository": {
                "commit": {"associatedPullRequests": {"edges": [{"node": None}]}}
            }
        }
    }

    def set_return_val(self, number, body):
        node = {"number": number, "body": body}
        self.val["data"]["repository"]["commit"]["associatedPullRequests"]["edges"][0][
            "node"
        ] = node

    def run_query(self, *args, **kwargs):
        return self.val


m = MockGraphQL()
p = PRs("fake token", "15five", "repoName")


@pytest.fixture(autouse=True)
def mock_graphql(mocker: MockFixture):
    global p

    mocker.patch("rocket_releaser.prs.GraphQL", return_value=m)

    yield

    # reset prs so tests dont affect eachother
    p = PRs("fake token", "15five", "repoName")


def test_pull_request_dicts_should_have_correct_content():
    m.set_return_val(12, "test")

    pull_request_dicts = p.pull_request_dicts(["fake sha"])
    assert len(pull_request_dicts) == 1
    assert pull_request_dicts[0]["number"] == 12
    assert pull_request_dicts[0]["body"] == "test"
    assert pull_request_dicts[0]["deploy_sha"] == "fake sha"
